local _G = _G
local date = _G.date
local MAJOR_VERSION = 'GraveLevelBar'
local MINOR_VERSION = type('20110507231424') == 'number' and '20110507231424' or date('%Y%m%d%H%M%S')
local MODULE_NAME = 'GuildXP'

local LibStub = _G.LibStub
local AceAddon = LibStub('AceAddon-3.0')
local addon = AceAddon:GetAddon(MAJOR_VERSION)

local module = addon:createXPModule(MODULE_NAME, 'AceEvent-3.0')

local UnitGetGuildXP = _G.UnitGetGuildXP
local GetGuildLevel  = _G.GetGuildLevel
local QueryGuildXP   = _G.QueryGuildXP
local min = min

-- start xp
function module:getStartXPMin()
    return 0
end

-- target level
function module:getTargetLevelMax()
    return MAX_GUILD_LEVEL
end

module.oldEnable = module.OnEnable
function module:OnEnable()
    self:oldEnable()
    self:RegisterEvent('GUILD_XP_UPDATE', 'Invalidate')
    -- TODO find proper events

    QueryGuildXP()
end

function module:Redraw()
    --addon:Debug("Guild XP Redraw")

    local xpOptions = self:getOptions()
    local barCache = self.cache

    local guildLevel = GetGuildLevel('player')
    local currentGlobalXP = self:getCurrentXP()
    -- update max settings if necessary
    if xpOptions.targetXP and currentGlobalXP > xpOptions.targetXP then
        xpOptions.targetXP = currentGlobalXP
    end
    local startGlobalXP = xpOptions.startXP
    local targetGlobalXP = xpOptions.targetXP or self:calcXpForLevel(MAX_GUILD_LEVEL)
    local remainingGlobalXP = self:getRemainingDailyXP()
    local currentGlobalXPRatio =   (currentGlobalXP  -startGlobalXP)/(targetGlobalXP-startGlobalXP)
    local remainingGlobalXPRatio = (remainingGlobalXP              )/(targetGlobalXP-startGlobalXP)
    -- create textures
    assert(barCache and barCache.frame)
    local frame = barCache.frame
    local right = frame:GetRight()
    local left = frame:GetLeft()
    if not (right and left) then
        self:Invalidate()
        return
    end
    local width = frame:GetRight() - frame:GetLeft()
    -- texture for curent xp
    local currentXPTexture = self:DrawPrimaryBar(currentGlobalXPRatio)

    -- texture for remaining xp
    barCache.remainingXPTexture = barCache.remainingXPTexture or frame:CreateTexture(nil,'ARTWORK')
    local remainingXPTexture = barCache.remainingXPTexture
    remainingXPTexture:ClearAllPoints()
    remainingXPTexture:SetPoint('TOP')
    remainingXPTexture:SetPoint('BOTTOM')
    remainingXPTexture:SetPoint('LEFT', currentXPTexture, 'RIGHT')
    local restedWidth = min(remainingGlobalXPRatio,1-currentGlobalXPRatio) * width
    remainingXPTexture:SetWidth(restedWidth)
    remainingXPTexture:SetTexture(0,0.5,0,1)

    self:DrawNumbers()

    --update options table
    self:updateOptionsTable()
end

function module:getCurrentXP()
    local xpInCurrentLevel, _, _, _ = UnitGetGuildXP('player')
    local level = GetGuildLevel('player')
    return xpInCurrentLevel + self:calcXpForLevel(level)
end

function module:getRemainingDailyXP()
    local _, _, dailyXP, maxDailyXP = UnitGetGuildXP('player')
    return maxDailyXP - dailyXP
end

function module:calcXpForLevel(level)
    assert(level <= MAX_GUILD_LEVEL,"level=" .. tostring(level))
    local sum = 0
    for i=1,level-1 do
        sum = sum + XP_PER_GUILD_LEVEL[i]
    end
    local fraction = level - floor(level)
    local xpForNextLevelLocal = XP_PER_GUILD_LEVEL[floor(level)]
    if xpForNextLevelLocal then
        return sum + fraction * xpForNextLevelLocal
    else
        return sum
    end

end

function module:calcLevelForXP(xp)
    local level = 1
    while xp > self:calcXpForLevel(level+1) do
        level = level + 1
    end
    local startXP = self:calcXpForLevel(level)
    assert(startXP <= xp,"level=" .. level .. " ,xp=" .. xp .. " ,startXP=" .. startXP)
    if startXP == xp then
        return level
    end
    local nextXP = self:calcXpForLevel(level+1)
    local fraction = (xp-startXP)/(nextXP-startXP)
    assert(0.0 <= fraction and fraction <= 1.0, "fraction=" .. fraction .. ", level=" .. level .. " ,xp=" .. xp .. " ,startXP=" .. startXP)
    return level + fraction
end

function module:fillTooltip(tooltip)
    QueryGuildXP()
    local currentGlobalXP = self:getCurrentXP()
    local targetGlobalXP = self:getOptions().targetXP or self:getTargetXPMax()
    local startGlobalXP = self:getOptions().startXP
    local deltaXP = targetGlobalXP-startGlobalXP
    local percent = (currentGlobalXP-startGlobalXP)/deltaXP
    local strPercent = format('%.2f%%', percent*100)
    local strCurrent = addon:formatNum(currentGlobalXP-startGlobalXP)
    local strDelta = addon:formatNum(deltaXP)
    tooltip:AddLine(strCurrent ..'/' .. strDelta .. '[' .. strPercent .. ']')
end

